% function fn_find_ir. Finds impulse response functions. 
% incorporates fn_find_ir_lagpi 
% clear all
% close all
 
% parameters
%b = 0.97; % beta
%k = 0.2; % kappa
%s = 0.3; % sigma
%xi = 0.1; % xi interest elasticity of money demand
%mc = 10; % money to output ratio

%itargt = [zeros(10,1);1;ones(10,1)]; %interest target
%itargt = [zeros(10,1);1;0.6.^(1:10)']; %interest target

%tim = (-10:10)';
%t0 = 11; % index of t = 0                        
function [pit,xt,pit_unex,xt_unex,r1,r2,l1,l2,l3,phi,ma] = fn_find_ir(itargt,imtargt,t0,b,k,s,xi,mc,rho,mamax);

l1=NaN; l2=NaN; l3=NaN; r1=NaN; r2=NaN;  % zero, not all are retured in all cases. 
phi = (b-rho)./(1+b*rho-2*rho);
% itargt is a vector consisting of the interest rate target path. 
%imtargt is a vector of interest rates on money. If zero, fill out to always be 0
if imtargt == 0; 
    imtargt = 0*itargt;
end; 
if rho > b; 
    disp('find_ir_lagpi: rho cannot exceed b. rho and b are ')
    disp([rho b])
end;


% two sided MA for inflation, then output %
%mamax = 50; % size of two sided moving average. Make it big. 

itargt  = [ones(mamax,1)*itargt(1)  ; itargt;  ones(mamax,1)*itargt(end)]; % pad target for ma
imtargt = [ones(mamax,1)*imtargt(1) ; imtargt; ones(mamax,1)*imtargt(end)]; % pad target for ma
zt = itargt - (s-xi)/s*mc*( [ (itargt(2:end)-imtargt(2:end)) - (itargt(1:end-1)-imtargt(1:end-1)) ; 0]);
zt_unex = zt;
zt_unex(t0+mamax-1) = itargt(t0+mamax-1) + 0; % they don't know 0 ahead of time
T = size(itargt,1); 

if rho == 0; % the rho > 0 formulas below give the same answer but nice to be explicit
    
    l1 = ((1+b+k*s)+ ((1+b+k*s)^2-4*b)^0.5)/2;
    l2 = ((1+b+k*s)- ((1+b+k*s)^2-4*b)^0.5)/2;
    
    ma = zeros(2*mamax+1,1);
    ma(1:mamax) = l2.^(mamax:-1:1)'; % forward. filter wants a one-sided ma
    ma(mamax+1) = 1;
    ma(mamax+2:end)=(1/l1).^(1:1:mamax)';
    ma = k*s/(l1-l2)*ma;
    
    ma_x = zeros(2*mamax+1,1);
    ma_x(1:mamax) = (1-b/l2)*l2.^(mamax:-1:1)'; % forward. filter wants a one-sided ma
    ma_x(mamax+1) = 1-b/l1;
    ma_x(mamax+2:end)=(1-b/l1)*(1/l1).^(1:1:mamax)';
    ma_x = k*s/(l1-l2)*ma_x;
end;

if (rho > 0 );
    if (rho < b);
        p = [ 1 -(1+phi+k*s+ phi*rho*(1-k*s))/phi (1+rho*(1+phi))/phi -rho/phi];
        r = roots(p);
        r12 = r(abs(r)<1);
        r1 = min(r12);
        r2 = max(r12);
        r3 = r(abs(r)>1); % uses r = 1/lambda to handle rho = 0, r=0, lambda = infty
        l3 = 1/r3;
        
        ma = zeros(2*mamax+1,1);
        ma(1:mamax) = l3.^(mamax:-1:1)'; % forward. filter wants a one-sided ma
        ma(mamax+1) = 1;
        ma(mamax+2:end) = (r1*(1-r2*l3)/(r1-r2))*(r1).^((1:1:mamax)')...
            -(r2*(1-r1*l3)/(r1-r2))*(r2).^((1:1:mamax)');
        ma = k*s*(1-phi*rho)/phi * l3/((1-l3*r1)*(1-l3*r2))*ma;
    end;
    if  (rho == b);
        phi = 0;
        r1 = ((1+b)-sqrt((1-b)^2-4*b*k*s))/(2*(1+k*s));
        r2 = ((1+b)+sqrt((1-b)^2-4*b*k*s))/(2*(1+k*s));
        l3 = 0;
        ma = zeros(2*mamax+1,1);
        %ma(1:mamax) = l3.^(mamax:-1:1)'; % forward. filter wants a one-sided ma
        ma(mamax+1) = 1;
        ma(mamax+2:end) = r1/(r1-r2)*r1.^((1:1:mamax)') - r2/(r1-r2)*r2.^((1:1:mamax)');
        ma = k*s/(1+k*s)*ma;
    end;
end
if abs(sum(ma)-1)>0.01; 
    disp('fn_find_ir: sum of ma is not one. likely increase maxma')
    disp(sum(ma)); 
end; 

% find inflation, output, by applying two sided filer
pit = filter(ma,1,zt);
pit_unex = filter(ma,1,zt_unex);
if rho == 0;
    xt = filter(ma_x,1,zt)/k;
    xt_unex = filter(ma_x,1,zt_unex)/k;
else;
    xt = ((1+phi*rho)/(1-phi*rho)*pit(2:end-1) - phi/(1-phi*rho)*pit(3:end) - rho/(1-phi*rho)*pit(1:end-2))/k;
    xt_unex = ((1+phi*rho)/(1-phi*rho)*pit_unex(2:end-1) - phi/(1-phi*rho)*pit_unex(3:end) - rho/(1-phi*rho)*pit_unex(1:end-2))/k;
    xt = [NaN; xt; NaN];
    xt_unex = [NaN; xt_unex; NaN];
end

% shift pi, x to be two sided filter
pit = [pit(mamax:end);NaN(mamax-1,1)]; % pi t+1 lines up with it
xt = [xt(mamax:end);NaN(mamax-1,1)]; 
pit_unex = [pit_unex(mamax:end);NaN(mamax-1,1)]; % pi t+1 lines up with it
xt_unex = [xt_unex(mamax:end);NaN(mamax-1,1)]; 

% get rid of early and late observations past ma length

itargt = itargt(mamax+1:end-mamax); % should be the same as before padding
imtargt = imtargt(mamax+1:end-mamax); % should be the same as before padding
pit = pit(mamax+1:end-mamax); 
xt = xt(mamax+1:end-mamax); 
pit_unex = pit_unex(mamax+1:end-mamax); 
xt_unex = xt_unex(mamax+1:end-mamax); 

% set to zero variables before the announcement date.

pit_unex(1:t0-1) = 0;
xt_unex(1:t0-1) = 0;
